///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Project: Jack the Uni-Psychle
//
// Author: Tim Newell
//
// License: GPL see LICENSE in main directory or /usr/share/JUP/ if you used the installer
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "MainWidget.h"
#include <qdirectpainter_qws.h>
#include <qpe/qpeapplication.h>
#include "PPlayerObject.h"
#include <stdio.h>
#include <qfile.h>
#include <qdir.h>

MainWidget::MainWidget(QWidget* parent,  const char* name)
                         :QWidget( parent, name) {

        setGeometry(0,0,240,320);
	
	//numBitmaps = 0;
	
	t = new QTimer( this );
        connect( t, SIGNAL(timeout()), SLOT(processLoop()) );
	inactive = false;
	timepassed = 0;
	
	hudLives = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/hud_life.png"), true);
	hudTires = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/hud_tire.png"), true);
	seconds = 0;
	minutes = 0;
	pushPix = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/push.png"), true);
	pullPix = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/pull.png"), true);
	menu = true;
	interval = 1;
	rollBackSelected1 = -1;
	rollBackSelected2 = -1;
	rollBackSelected3 = -1;
	rollBackArrows = NULL;
	inGame = false;
	paused = false;
	gameend = false;
	slide = false;
	
	levelSystem = new PLevelSystem(this);
    
    	//Menu stuff
	introdone = false;
	pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/ds.jpg"));
	loadscreen = false;
	selected = 0;
	tmprofile = NULL;
        
	for (BYTE i = 0; i < 10; i++) 
		keys[i] = false;
		
	QPEApplication::grabKeyboard();
	frameRateTimer.start();
	tstablizer.start();
	
	//Options Menu
	optionsMenu = false;
	strcpy(password,"_ _ _ _ _ _");
	strcpy(passOk,"");
	
	//declare our back buffer
	PBackBuffer.create(240,320);
	
	fontOne.create(QPixmap("/opt/QtPalmtop/pics/jack/images/fonts/font_small.png"),QColor(0,0,0));
	fontTwo.create(QPixmap("/opt/QtPalmtop/pics/jack/images/fonts/font_large.png"),QColor(0,0,0));
	
	//Music
	music = NULL;
	MusicOn = true;
	SoundOn = true;
	
	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
	int audio_channels = 2;
	int audio_buffers = 512;

  
  	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
			printf("Unable to open audio!\n");
			exit(1);
	}
	
	//default volumes
	soundVolume = 95;
	musicVolume = 85;
	
	sounds[0] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Wheel.wav");
	sounds[1] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/PsyAttack.wav");
	sounds[2] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Jump.wav");
	sounds[3] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Attack.wav");
	sounds[4] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/ExtraLife.wav");
	sounds[5] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Pause.wav");
	sounds[6] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/BeepMenuV.wav");
	sounds[7] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/BeepMenuH.wav");
	sounds[8] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/BeepMenuA.wav");
	sounds[9] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Timer10.wav");
	sounds[10] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/TimerOut.wav");
	sounds[11] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Pod.wav");
	sounds[12] = NULL;
	sounds[13] = Mix_LoadWAV("/opt/QtPalmtop/sounds/jack/sounds/Bomb.wav");
	
	//create our save dir
	QDir d = QDir::homeDirPath();
	if (!d.cd(".jack")) {
		d.mkdir(".jack");
	}

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
MainWidget::~MainWidget() {

	if (music != NULL) {
		Mix_HaltMusic();
		Mix_FreeMusic(music);
		music = NULL;
	}
	
	for (int i = 0; i < 14; i++) {
		Mix_FreeChunk(sounds[i]);
	}
	
	Mix_CloseAudio();
	
	if (t != NULL) {
		delete t;
		t = NULL;
	}
	
	if (hudLives != NULL) {
		delete hudLives;
		hudLives = NULL;
	}
	
	if (hudTires != NULL) {
		delete hudTires;
		hudTires = NULL;
	}
	
	if (pushPix != NULL) {
		delete pushPix;
		pushPix = NULL;
	}
	
	if (pullPix != NULL) {
		delete pullPix;
		pullPix = NULL;
	}
	
	if (pic != NULL) {
		delete pic;
		pic = NULL;
	}
	
	if (tmprofile != NULL) {
		delete tmprofile;
		tmprofile = NULL;
	}
	
	if (levelSystem != NULL) {
		delete levelSystem;
		levelSystem = NULL;
	}
	
	if (rollBackArrows != NULL) {
		delete rollBackArrows;
		rollBackArrows = NULL;
	}
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//some fullscreencode
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::resizeEvent(QResizeEvent *) {
      if(size() != qApp->desktop()->size()) {
              // Widget is not the correct size, so do the fullscreen magic
             enableFullscreen();
      }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//some fullscreencode
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::focusInEvent(QFocusEvent *) {
      
      if (inactive == true) {
	      inactive = false;
      }
      
      // Always do it here, no matter the size.
      enableFullscreen();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//popups cause loss of focus.. need to regain
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::focusOutEvent ( QFocusEvent *) {
	inactive = true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//some fullscreencode
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::enableFullscreen() {
      // Make sure size is correct
      setFixedSize(qApp->desktop()->size());
      // This call is needed because showFullScreen won't work
      // correctly if the widget already considers itself to be fullscreen.
      showNormal();
      // This is needed because showNormal() forcefully changes the window
      // style to WSTyle_TopLevel.
      reparent(0, WStyle_Customize | WStyle_NoBorder, QPoint(0,0));
      // Enable fullscreen.
      showFullScreen();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::keyReleaseEvent( QKeyEvent *e ) {
	
		switch (e->key()) {
			case Key_Left:
				keys[KEY_LEFT] = false;
				break;
			case Key_Right:
				keys[KEY_RIGHT] = false;
				break;
			case Key_F12:
				keys[KEY_F12] = false;
				break;
			case Key_Escape:
				keys[KEY_ESCAPE] = true;
				break;
			case Key_F33:
				keys[KEY_F33] = false;
				break;
			case Key_Enter:
				keys[KEY_ENTER] = false;
				break;
			case Key_F8:
				keys[KEY_F8] = false;
				break;
			case Key_F13:
				keys[KEY_F13] = true;
				break;
		}
	 
	
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::keyPressEvent( QKeyEvent *e ) {
	
	if (menu) {
	    
	    //check to see if introdone is false, if so then they pressed a key to remove the discord logo.. display the main menu
            if (!introdone) {
		    introdone = true;
		    LoadLevelMusic();
		    if (pic != NULL) {
		         delete pic;
		         pic = NULL;
	            }
		    pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
	    } else if (!loadscreen && !optionsMenu && !inGame && !gameend) {
		    switch (e->key()) {

			    case Key_F33:
		            case Key_Enter:
				   levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				   startGame();
			           break;
			    case Key_Space:
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				    loadOptions();
				    break;
		 	    case Key_Escape:
		            case Key_F12:
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				    quitGame();
				    break;
			    
		    }
	    
	    } else if (loadscreen) {
		
		    switch (e->key()) {

			    case Key_Up:
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				    selected--;
				    if (selected < 1)
					    selected = 1;
				    break;
		            case Key_Down:
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				    selected++;
				    if (selected > 3)
					    selected = 3;
			           break;
			    case Key_Left:
			    	    if (selected == 1) {
					    if (rollBackSelected1 == -1)
						    break;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected1--;
					    if (rollBackSelected1 < 0)
						    rollBackSelected1 = 0;
				    } else if (selected == 2) {
					    if (rollBackSelected2 == -1)
						    break;
				            levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected2--;
					    if (rollBackSelected2 < 0)
						    rollBackSelected2 = 0;
				    } else if (selected == 3) {
					    if (rollBackSelected3 == -1)
						    break;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected3--;
					    if (rollBackSelected3 < 0)
						    rollBackSelected3 = 0;
				    }
				    break;
			    case Key_Right:
				    if (selected == 1) {
					    
					    if (rollBackSelected1 == -1)
						    break;
				            levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected1++;
					    if (rollBackSelected1 > tmprofile->number-1)
						    rollBackSelected1 = tmprofile->number-1;
				    } else if (selected == 2) {
					    if (rollBackSelected2 == -1)
						    break;
				            levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected2++;
					    if (rollBackSelected2 > tmprofile->number2-1)
						    rollBackSelected2 = tmprofile->number2-1;
				    } else if (selected == 3) {
					    if (rollBackSelected3 == -1)
						    break;
				            levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected3++;
					    if (rollBackSelected3 > tmprofile->number3-1)
						    rollBackSelected3 = tmprofile->number3-1;
				    }
				    break;
			    case Key_F33:
		            case Key_Enter:
			    	    if (selected >=1 && selected <= 3) {
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
					    loadGame(selected,1);
				    }
				    break;
			    case Key_Escape:
		            case Key_F12:
			    	    if (selected >=1 && selected <= 3) {
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
					    loadGame(selected,0);
				    }
				    break;
		    }
		
	    } else if (optionsMenu == true) {
	    
	    	switch (e->key()) {
		    case Key_Escape:
		    case Key_F12:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
			    returnMenu();
			    break;
		    case Key_Up:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
			    selected--;
			    if (selected < 1)
				    selected = 1;
			    break;
		    case Key_Down:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
			    selected++;
			    if (selected > 3)
				    selected = 3;
			    break;
		    case Key_Left:
		    	    if (selected == 1) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				    musicVolume -= 5;
				    if (musicVolume < 0)
					    musicVolume = 0;
				    Mix_VolumeMusic(musicVolume);
			    
			    } else if (selected == 2) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				    soundVolume -= 5;
				    if (soundVolume < 0)
					    soundVolume = 0;
			    }
			    break;
		    case Key_Right:
			    if (selected == 1) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				    musicVolume += 5;
				    if (musicVolume > MIX_MAX_VOLUME)
					    musicVolume = MIX_MAX_VOLUME;
				    Mix_VolumeMusic(musicVolume);
			    
			    } else if (selected == 2) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				    soundVolume += 5;
				    if (soundVolume > MIX_MAX_VOLUME)
					    soundVolume = MIX_MAX_VOLUME;
			    
			    }
			    break;
		    case Key_Space:
			    if (selected == 1) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				    MusicOn = !MusicOn;
				    if (MusicOn)
					LoadLevelMusic();
				else
					StopLevelMusic(true);
			    } else if (selected == 2) {
				    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				    SoundOn = !SoundOn;
			    }
			    break;
		    case Key_0:
		    	    PasswordCheck('0');
			    break;
		    case Key_1:
			    PasswordCheck('1');
			    break;
		    case Key_2:
			    PasswordCheck('2');
			    break;
		    case Key_3:
			    PasswordCheck('3');
			    break;
		    case Key_4:
			    PasswordCheck('4');
			    break;
		    case Key_5:
			    PasswordCheck('5');
			    break;
		    case Key_6:
			    PasswordCheck('6');
			    break;
		    case Key_7:
			    PasswordCheck('7');
			    break;
		    case Key_8:
			    PasswordCheck('8');
			    break;
		    case Key_9:
			    PasswordCheck('9');
			    break;
		}
	    } else if (inGame == true) {
		    switch (e->key()) {
		       case Key_F33:
		       case Key_Enter:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
			    menu = false;
			    inGame = false;
			    LoadLevelMusic();
		            if (pic != NULL) {
				    delete pic;
				    pic = NULL;
			    }
			    return;
			    break;
		       case Key_Space:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
			    loadOptions();
			    break;
		       case Key_Escape:
		       case Key_F12:
			    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
			    levelSystem->cleanup();
			    if (pic != NULL) {
				    delete pic;
				    pic = NULL;
			    }
			    pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
			    inGame = false;
			    break;
		    }
		    
	    } else if (gameend == true) {
		    switch (endscreen) {
			     case 5:
				if (tstablizer.elapsed() < 3000)
					return;
				if (pic != NULL) {
				    delete pic;
				    pic = NULL;
				}
				pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/demo2.jpg"));
				levelSystem->cleanup();
				endscreen = 1;
				tstablizer.restart();
				break;
			    case 1:
				if (tstablizer.elapsed() < 3000)
					return;
				if (pic != NULL) {
				    delete pic;
				    pic = NULL;
				}
				pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
				gameend = false;
				tstablizer.restart();
				break;
		        }
	    }
	    
    } else {
	    

	switch (e->key()) {
		case Key_Left:
				keys[KEY_LEFT] = true;
				break;
		case Key_Right:
				keys[KEY_RIGHT] = true;
				break;
		case Key_F12:
				keys[KEY_F12] = true;
				break;
		case Key_Escape:
				keys[KEY_ESCAPE] = true;
				break;
		case Key_F33:
				keys[KEY_F33] = true;
				break;
		case Key_Enter:
				keys[KEY_ENTER] = true;
				break;
		case Key_F8:
				keys[KEY_F8] = true;
				break;
		case Key_F13:
				keys[KEY_F13] = true;
				break;
	}
    }
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::mousePressEvent ( QMouseEvent *e ) {
	
	if (!menu)
		return;
	
	if (!introdone) {
             introdone = true;
	     LoadLevelMusic();
     	     if (pic != NULL) {
		delete pic;
		pic = NULL;
             }
	     pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
        } else if (loadscreen == false && introdone == true && optionsMenu == false && inGame == false && gameend == false) {
		if (e->x() >= 147 && e->x() <= 235) {  //its one of the buttons
		
			if (e->y() >= 202 && e->y() <= 229) {  //they clicked the first button
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				startGame();
			} else if (e->y() >= 242 && e->y() <= 269) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				loadOptions();
			} else if (e->y() >= 279 && e->y() <= 306) { //they clicked the second button
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				quitGame();
			}
		
		}
        } else if (loadscreen == true) {
		
		if (e->y() >= 2 && e->y() <= 22) {
			
			if (e->x() >= 40 && e->x() <= 60) {
				
				   if (selected == 1) {
					    if (rollBackSelected1 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected1--;
					    if (rollBackSelected1 < 0)
						    rollBackSelected1 = 0;
				    } else if (selected == 2) {
					    if (rollBackSelected2 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected2--;
					    if (rollBackSelected2 < 0)
						    rollBackSelected2 = 0;
				    } else if (selected == 3) {
					    if (rollBackSelected3 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected3--;
					    if (rollBackSelected3 < 0)
						    rollBackSelected3 = 0;
				    }
				
			} else if (e->x() >= 172 && e->x() <= 192) {

				if (selected == 1) {
					    if (rollBackSelected1 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected1++;
					    if (rollBackSelected1 > tmprofile->number-1)
						    rollBackSelected1 = tmprofile->number-1;
				    } else if (selected == 2) {
					    if (rollBackSelected2 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected2++;
					    if (rollBackSelected2 > tmprofile->number2-1)
						    rollBackSelected2 = tmprofile->number2-1;
				    } else if (selected == 3) {
					    if (rollBackSelected3 == -1)
						    return;
					    levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
					    rollBackSelected3++;
					    if (rollBackSelected3 > tmprofile->number3-1)
						    rollBackSelected3 = tmprofile->number3-1;
				    }
			}
			
		}
		
		if (e->x() >= 2 && e->x() <= 237) {
		
			if (e->y() >= 23 && e->y() <= 65) {  //they clicked the first profile
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				selected = 1;
			} else if (e->y() >= 68 && e->y() <= 110) {  //they clicked the second profile
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				selected = 2;
			} else if (e->y() >= 113 && e->y() <= 155) {  //they clicked the third profile
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				selected = 3;
			}
		}
		
		if (e->y() >= 282 && e->y() <= 310) {
			if (e->x() >= 4 && e->x() <= 87) {  //continue
				if (selected != 0)
					levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
					loadGame(selected,0);
			} else if (e->x() >= 157 && e->x() <= 235) {  //new
				if (selected != 0)
					levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
					loadGame(selected,1);
			}
		}
		
		
	} else if (optionsMenu == true) {
		
		selected = 0;
		if (e->y() >= 163 && e->y() <= 191) {
			
			if (e->x() >= 8 && e->x() <= 136) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				returnMenu();
			}
		}
		
		if (e->y() >= 37 && e->y() <= 55) {
			
			if (e->x() >= 10 && e->x() <= 28) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				MusicOn = !MusicOn;
				if (MusicOn)
					LoadLevelMusic();
				else
					StopLevelMusic(true);
			} else if (e->x() >= 120 && e->x() <= 220) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				musicVolume = (int)(((float)((e->x() - 120) / 100.0f)) * 128);
				Mix_VolumeMusic(musicVolume);
			}
		}
		
		if (e->y() >= 75 && e->y() <= 93) {
			
			if (e->x() >= 10 && e->x() <= 28) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				SoundOn = !SoundOn;
			} else if (e->x() >= 120 && e->x() <= 220) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuH);
				soundVolume = (int)(((float)((e->x() - 120) / 100.0f)) * 128);
			}
		}
		if (e->y() >= 117 && e->y() <= 141) {
			if (e->x() >= 120 && e->x() <= 220) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuV);
				selected = 3;
			}
		}
	} else if (inGame == true) {
		if (e->x() >= 46 && e->x() <= 188) {  //its one of the buttons
		
			if (e->y() >= 34 && e->y() <= 61) {  //they clicked the first button
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				 inGame = false;
				 menu = false;
				 LoadLevelMusic();
				 if (pic != NULL) {
					 delete pic;
					 pic = NULL;
				 }
				 return;
			} else if (e->y() >= 73 && e->y() <= 101) {
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				loadOptions();
			} else if (e->y() >= 110 && e->y() <= 138) { //they clicked the second button
				levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
				levelSystem->cleanup();
				if (pic != NULL) {
				    delete pic;
				    pic = NULL;
			        }
				pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
			        inGame = false;
			}
		
		}
	} else if (gameend == true) {
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Render the graphics to a pixmap then slap it onscreen
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::render() {
           
	   if (inactive == true)
		   return;
	   
	   ////////////////////////////////////////////////////////////////////////////////////////////////////////
	   PBackBuffer.fill(QColor(0,0,0));
	   ZSurface tmp;
	   
	   //draw the menu
	   if (menu) {
			
		ZSurface::bitBlit(&PBackBuffer,0,0,pic);
	   
	 	if (loadscreen && tmprofile != NULL) {
			
			//left arrow
			if (((selected == 1) && (rollBackSelected1 != -1) && (rollBackSelected1 > 0)) ||
			    ((selected == 2) && (rollBackSelected2 != -1) && (rollBackSelected2 > 0)) ||
			    ((selected == 3) && (rollBackSelected3 != -1) && (rollBackSelected3 > 0))) {
				    
				    ZSurface::bitBlit(&PBackBuffer,40,2,rollBackArrows->image(1));
			} else {
				ZSurface::bitBlit(&PBackBuffer,40,2,rollBackArrows->image(3));
			}
			
			//right arrow
			if (((selected == 1) && (rollBackSelected1 != -1) && (rollBackSelected1 < (tmprofile->number-1))) ||
			    ((selected == 2) && (rollBackSelected2 != -1) && (rollBackSelected2 < (tmprofile->number2-1))) ||
			    ((selected == 3) && (rollBackSelected3 != -1) && (rollBackSelected3 < (tmprofile->number3-1)))) {
				    ZSurface::bitBlit(&PBackBuffer,172,2,rollBackArrows->image(0));
			} else {
				ZSurface::bitBlit(&PBackBuffer,172,2,rollBackArrows->image(2));
			}

		    if (rollBackSelected1 == -1) {
			if (selected == 1) {
				short tt = tmp.drawText(137,6, "Selected",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),34, "Selected",&fontOne,QColor(233,247,179));
			} else {
				short tt = tmp.drawText(137,6, "Empty",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),34, "Empty",&fontOne,QColor(233,247,179));
			}
		    } else {
			//Profile 1
			ZSurface::bitBlitAlpha(&PBackBuffer,3,24,hudLives);
			ZSurface::bitBlitAlpha(&PBackBuffer,176,24,hudTires);
			PBackBuffer.drawText(63,34, QString::number(tmprofile->lives[rollBackSelected1]),&fontOne,QColor(233,247,179));
			short tt = tmp.drawText(137,6, QString::number(tmprofile->tires[rollBackSelected1]),&fontOne,QColor(233,247,179));
			PBackBuffer.drawText(175-tt,34, QString::number(tmprofile->tires[rollBackSelected1]),&fontOne,QColor(233,247,179));

			if (selected == 1) {
				short tt = tmp.drawText(137,6, "* " + QString::number(tmprofile->level[rollBackSelected1]+1) + " *",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),34, "* " + QString::number(tmprofile->level[rollBackSelected1]+1) + " *",&fontOne,QColor(233,247,179));
			} else {
				short tt = tmp.drawText(137,6, "lvl " + QString::number(tmprofile->level[rollBackSelected1]+1),&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),34, "lvl " + QString::number(tmprofile->level[rollBackSelected1]+1),&fontOne,QColor(233,247,179));
			}
		    }
		
		    if (rollBackSelected2 == -1) {
			if (selected == 2) {
				short tt = tmp.drawText(137,6, "Selected",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),79, "Selected",&fontOne,QColor(233,247,179));
			} else {
				short tt = tmp.drawText(137,6, "Empty",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),79, "Empty",&fontOne,QColor(233,247,179));
			}
	   	    } else {
			//Profile 2
			ZSurface::bitBlitAlpha(&PBackBuffer,3,69,hudLives);
			ZSurface::bitBlitAlpha(&PBackBuffer,176,69,hudTires);
			PBackBuffer.drawText(63,79, QString::number(tmprofile->lives2[rollBackSelected2]),&fontOne,QColor(233,247,179));
			short tt = tmp.drawText(137,6, QString::number(tmprofile->tires2[rollBackSelected2]),&fontOne,QColor(233,247,179));
			PBackBuffer.drawText(175-tt,79, QString::number(tmprofile->tires2[rollBackSelected2]),&fontOne,QColor(233,247,179));
		
			if (selected == 2) {
				short tt = tmp.drawText(137,6, "* " + QString::number(tmprofile->level2[rollBackSelected2]+1) + " *",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),79, "* " + QString::number(tmprofile->level2[rollBackSelected2]+1) + " *",&fontOne,QColor(233,247,179));
				
			} else {
				short tt = tmp.drawText(137,6, "lvl " + QString::number(tmprofile->level2[rollBackSelected2]+1),&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),79, "lvl " + QString::number(tmprofile->level2[rollBackSelected2]+1),&fontOne,QColor(233,247,179));
			}
			
                    }
		
		    if (rollBackSelected3 == -1) {
			if (selected == 3) {
				short tt = tmp.drawText(137,6, "Selected",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),124, "Selected",&fontOne,QColor(233,247,179));
			} else {
				short tt = tmp.drawText(137,6, "Empty",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),124, "Empty",&fontOne,QColor(233,247,179));
			}
		    } else {
			//Profile 3
			ZSurface::bitBlitAlpha(&PBackBuffer,3,114,hudLives);
			ZSurface::bitBlitAlpha(&PBackBuffer,176,114,hudTires);
			PBackBuffer.drawText(63,124, QString::number(tmprofile->lives3[rollBackSelected3]),&fontOne,QColor(233,247,179));
			short tt = tmp.drawText(137,6, QString::number(tmprofile->tires3[rollBackSelected3]),&fontOne,QColor(233,247,179));
			PBackBuffer.drawText(175-tt,124, QString::number(tmprofile->tires3[rollBackSelected3]),&fontOne,QColor(233,247,179));
			
			if (selected == 3) {
				short tt = tmp.drawText(137,6, "* " + QString::number(tmprofile->level3[rollBackSelected3]+1) + " *",&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),124, "* " + QString::number(tmprofile->level3[rollBackSelected3]+1) + " *",&fontOne,QColor(233,247,179));
			} else {
				short tt = tmp.drawText(137,6, "lvl " + QString::number(tmprofile->level3[rollBackSelected3]+1),&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),124, "lvl " + QString::number(tmprofile->level3[rollBackSelected3]+1),&fontOne,QColor(233,247,179));
			}
		    }
	   
		} else if (optionsMenu == true) {
		
			//Music CheckBox
			PBackBuffer.drawLine(10, 37, 10+18, 37, QColor(233,247,179));
			PBackBuffer.drawLine(10, 37, 10, 37+18, QColor(233,247,179));
			PBackBuffer.drawLine(10+18, 37, 10+18, 37+18, QColor(233,247,179));
			PBackBuffer.drawLine(10, 37+18, 10+18, 37+18, QColor(233,247,179));
			if (selected == 1)
				PBackBuffer.drawText(33,37, "Music*",&fontOne,QColor(233,247,179));
			else
				PBackBuffer.drawText(33,37, "Music",&fontOne,QColor(233,247,179));
			
			if (MusicOn == true) {  //render a check
				ZSurface::fillRect(&PBackBuffer, 12, 40, 15, 13, QColor(233,247,179));
				ZSurface::fillRect(&PBackBuffer, 120, 37, (int)(((float)musicVolume/MIX_MAX_VOLUME)*100), 18, QColor(233,247,179));  //colored bar
			}
			//Music Slider
			PBackBuffer.drawLine(120, 37, 120+100, 37, QColor(233,247,179));
			PBackBuffer.drawLine(120, 37, 120, 37+18, QColor(233,247,179));
			PBackBuffer.drawLine(120+100, 37, 120+100, 37+18, QColor(233,247,179));
			PBackBuffer.drawLine(120, 37+18, 120+100, 37+18, QColor(233,247,179));
			
			//Sound Checkbox
			PBackBuffer.drawLine(10, 75, 10+18, 75, QColor(233,247,179));
			PBackBuffer.drawLine(10, 75, 10, 75+18, QColor(233,247,179));
			PBackBuffer.drawLine(10+18, 75, 10+18, 75+18, QColor(233,247,179));
			PBackBuffer.drawLine(10, 75+18, 10+18, 75+18, QColor(233,247,179));
			if (selected == 2)
				PBackBuffer.drawText(33,75, "Sound*",&fontOne,QColor(233,247,179));
			else
				PBackBuffer.drawText(33,75, "Sound",&fontOne,QColor(233,247,179));
			
			if (SoundOn == true) {  //render a check
				ZSurface::fillRect(&PBackBuffer, 12, 78, 15, 13, QColor(233,247,179));
				ZSurface::fillRect(&PBackBuffer, 120, 75, (int)(((float)soundVolume/MIX_MAX_VOLUME)*100), 18, QColor(233,247,179));  //colored bar
			}
			
			//Sound Slider
			PBackBuffer.drawLine(120, 75, 120+100, 75, QColor(233,247,179));
			PBackBuffer.drawLine(120, 75, 120, 75+18, QColor(233,247,179));
			PBackBuffer.drawLine(120+100, 75, 120+100, 75+18, QColor(233,247,179));
			PBackBuffer.drawLine(120, 75+18, 120+100, 75+18, QColor(233,247,179));
			
			//CheatBox
			if (selected == 3)
				PBackBuffer.drawText(10,125, "Secret Code*",&fontOne,QColor(233,247,179));
			else
				PBackBuffer.drawText(10,125, "Secret Code",&fontOne,QColor(233,247,179));
			short tt = tmp.drawText(125,125, password,&fontOne,QColor(233,247,179));
			PBackBuffer.drawText(120 + (50 - (tt >> 1)),125, password,&fontOne,QColor(233,247,179));
			PBackBuffer.drawLine(120, 122, 120+100, 122, QColor(233,247,179));
			PBackBuffer.drawLine(120, 122, 120, 122+24, QColor(233,247,179));
			PBackBuffer.drawLine(120+100, 122, 120+100, 122+24, QColor(233,247,179));
			PBackBuffer.drawLine(120, 122+24, 120+100, 122+24, QColor(233,247,179));
			if (strcmp(passOk,"") != 0) {
				short tt = tmp.drawText(120,98, passOk,&fontOne,QColor(233,247,179));
				PBackBuffer.drawText(120 - (tt >> 1),98, passOk,&fontOne,QColor(233,247,179));
			}
		
		} else if (gameend == true) {
		}
	  
	   } else {
	   
		ZSurface::bitBlitAlpha(&PBackBuffer,0,0,hudLives);
		ZSurface::bitBlitAlpha(&PBackBuffer,180,0,hudTires);
		
		if (levelSystem->getTimerOff()) {
			PBackBuffer.drawText(4,293, "Time? who cares?",&fontOne,QColor(233,247,179));
		} else {
		
			if (levelSystem->getLevelTimer() == -1) 
				PBackBuffer.drawText(4,293, "Time Elapsed: " + ((minutes < 10)?"0" + QString::number(minutes):QString::number(minutes)) + ":" + ((seconds < 10)?"0" + QString::number(seconds):QString::number(seconds)),&fontOne,QColor(233,247,179));
			else
				PBackBuffer.drawText(4,293, "Time Left: " + ((minutes < 10)?"0" + QString::number(minutes):QString::number(minutes)) + ":" + ((seconds < 10)?"0" + QString::number(seconds):QString::number(seconds)),&fontOne,QColor(233,247,179));
		}
		
		//draw FPS
		//PBackBuffer.drawText(170,293, QString::number(fps),&fontOne,QColor(233,247,179));
		
		PBackBuffer.drawText(65,2, QString::number(levelSystem->getPlayer()->getLives()),&fontTwo,QColor(233,247,179));
		short t = tmp.drawText(137,6, QString::number(levelSystem->getPlayer()->getNumTires()),&fontTwo,QColor(233,247,179));
		PBackBuffer.drawText(177 - t,2, QString::number(levelSystem->getPlayer()->getNumTires()),&fontTwo,QColor(233,247,179));
	   
	   	//render the game tiles
		levelSystem->render(PBackBuffer);
		
	   	if (levelSystem->getPlayer()->getPush()) {
			if (levelSystem->getPlayer()->getGoingRight())
				ZSurface::bitBlitAlpha(&PBackBuffer,levelSystem->getPlayer()->getX() + levelSystem->getPlayer()->getWidth() + 20,levelSystem->getPlayer()->getY() + (levelSystem->getPlayer()->getHeight() << 1) - (levelSystem->getPlayer()->getHeight() << 1),pushPix);
			else
				ZSurface::bitBlitAlpha(&PBackBuffer,levelSystem->getPlayer()->getX() - 20 - pullPix->width(), levelSystem->getPlayer()->getY() + (levelSystem->getPlayer()->getHeight() << 1) - (levelSystem->getPlayer()->getHeight() << 1), pullPix);
		}
			
	   	if (levelSystem->getPlayer()->getDead() == 1) {
			short t = tmp.drawText(137,6, levelSystem->getPlayer()->getDeadText(),&fontTwo,QColor(233,247,179));
			if ((levelSystem->getCurrentLevel() > 1 && levelSystem->getCurrentLevel() < 5) || levelSystem->getCurrentLevel() == 12 || levelSystem->getCurrentLevel() == 13 || levelSystem->getCurrentLevel() == 17)
				PBackBuffer.drawText(120-(t >> 1),100, levelSystem->getPlayer()->getDeadText(),&fontTwo,QColor(102,198,101));
			else
				PBackBuffer.drawText(120-(t >> 1),100, levelSystem->getPlayer()->getDeadText(),&fontTwo,QColor(233,247,179));
		}
		
		if (paused) {
			short t = tmp.drawText(137,6, "Paused",&fontTwo,QColor(233,247,179));
			if ((levelSystem->getCurrentLevel() > 1 && levelSystem->getCurrentLevel() < 5) || levelSystem->getCurrentLevel() == 12 || levelSystem->getCurrentLevel() == 13 || levelSystem->getCurrentLevel() == 17) {
				PBackBuffer.drawText(120-(t >> 1),100, "Paused",&fontTwo,QColor(102,198,101)); 
			} else {
				PBackBuffer.drawText(120-(t >> 1),100, "Paused",&fontTwo,QColor(233,247,179));
			}
		}

	   	if (levelSystem->getLevelFinished()) {
			short t = tmp.drawText(137,6, "Congratulations!",&fontTwo,QColor(233,247,179));
			QString tmptime = "Time: " + ((minutes < 10)?"0" + QString::number(minutes):QString::number(minutes)) + ":" + ((seconds < 10)?"0" + QString::number(seconds):QString::number(seconds));
			short tt = tmp.drawText(137,6, tmptime,&fontTwo,QColor(233,247,179));
			if ((levelSystem->getCurrentLevel() > 1 && levelSystem->getCurrentLevel() < 5) || levelSystem->getCurrentLevel() == 12 || levelSystem->getCurrentLevel() == 13 || levelSystem->getCurrentLevel() == 17) {
				PBackBuffer.drawText(120-(t >> 1),100, "Congratulations!",&fontTwo,QColor(102,198,101)); 
				PBackBuffer.drawText(120-(tt >> 1),200, tmptime,&fontTwo,QColor(102,198,101));
			} else {
				PBackBuffer.drawText(120-(t >> 1),100, "Congratulations!",&fontTwo,QColor(233,247,179));
				PBackBuffer.drawText(120-(tt >> 1),200, tmptime,&fontTwo,QColor(233,247,179));
			}
		}
		
		if (levelSystem->getLevelStarted()) {
			short t = tmp.drawText(137,6, levelSystem->getLevelText(),&fontTwo,QColor(233,247,179));
			if ((levelSystem->getCurrentLevel() > 1 && levelSystem->getCurrentLevel() < 5) || levelSystem->getCurrentLevel() == 12 || levelSystem->getCurrentLevel() == 13 || levelSystem->getCurrentLevel() == 17) {
				PBackBuffer.drawText(120-(t >> 1),100, levelSystem->getLevelText(),&fontTwo,QColor(102,198,101)); 
			} else {
				PBackBuffer.drawText(120-(t >> 1),100, levelSystem->getLevelText(),&fontTwo,QColor(233,247,179));
			}
		}
	   
	   }
	   
           ////////////////////////////////////////////////////////////////////////////////////////////////////////////
	  // if (writeBitmap) {
		 //  record(PBackBuffer);
		//   writeBitmap = false;
	   //} else {
		//   writeBitmap = true;
	   //}
           QDirectPainter dp(this);  //create a direct painter to the screen 
	   ZSurface::bitBlit(&dp,&PBackBuffer,ZSurface::Z0);
           ///////////////////////////////////////////////////////////////////////////////////////////////////////////
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  If the window throws a repaint event then make sure we update as well
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::paintEvent( QPaintEvent *q ) {
     QWidget::paintEvent(q);
     
     //start our looping timer
     t->start( 0, FALSE );

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::processLoop() {			

	if (paused) {
		if (pauseTimer.elapsed() > 250) {
			checkInput();
		}
		return;
	}
	
	if (!inactive && !menu && !levelSystem->getLoading()) {
		
		//grab time passed since last frame
		timepassed = frameRateTimer.elapsed();
	
		if (timepassed < 30)
			return;
	
		frameRateTimer.restart();
		//fps = 1000 / timepassed;
		
		levelSystem->setLoading(true);
		
		if (levelSystem->getPlayer()->getDead() == 0 && !levelSystem->getLevelFinished() && !levelSystem->getLevelStarted()) {
		
			checkInput();
			
			if (levelSystem->getPlayer()->getJump() && levelSystem->getPlayer()->getDead() == 0) {
				for (BYTE i = 0; i < 2; i++)
					levelSystem->getPlayer()->applyGravity();
				if (levelSystem->getPlayer()->getJumpedDistance() >= 80) {    //is 120 for PC is 80 for Zaurus
					levelSystem->getPlayer()->setJump(false);
				}
			} else if (!levelSystem->getPlayer()->getJump() && levelSystem->getPlayer()->getDead() == 0 && levelSystem->getPlayer()->getJumpedDistance() > 0) {
				for (BYTE i = 0; i < 2; i++)
					levelSystem->getPlayer()->applyGravity();
			} else {
				
				for (BYTE i = 0; i < 2; i++)  //is 2 for PC.. 4 for Zaurus
					levelSystem->getPlayer()->applyGravity();
				
			}
			
			if (tstablizer.elapsed() > 150 && !levelSystem->getLevelFinished() && !levelSystem->getLevelStarted()) { 
				levelSystem->calculateAI();
				tstablizer.restart();
			}
			
			
		
		} else if (levelSystem->getPlayer()->getDead() == 0 && levelSystem->getLevelFinished()) {
			StopLevelMusic();
	     		if (tstablizer.elapsed() > 2000) {
				levelSystem->setLevelFinished(false);
				for (short i = 0; i < 10; i++) 
					keys[i] = false;
				if (levelSystem->getCurrentLevel() == 5) {
					levelSystem->gameEnd();
					return;
				} else {
					levelSystem->ChangeLevels();
					LoadLevelMusic();
					tstablizer.restart();
				}
			}
		} else if(levelSystem->getLevelStarted() == true) {
			for (BYTE i = 0; i < 2; i++)
				levelSystem->getPlayer()->applyGravity();
			if (seconds > 0)
				levelSystem->getLevelTime().restart();
			if (tstablizer.elapsed() > 2000) {
				levelSystem->setLevelStarted(false);
				for (short i = 0; i < 10; i++) {
					keys[i] = false;
				}
				tstablizer.restart();
				levelSystem->getLevelTime().restart();
			}
		} else {
			StopLevelMusic();
			levelSystem->getPlayer()->processDeadAI();
			if (tstablizer.elapsed() > 2000) {
				levelSystem->getPlayer()->setDead(0);
				levelSystem->getPlayer()->setDeadText("");
				if (levelSystem->getPlayer()->getLives() == 0) {
					levelSystem->cleanup();
					if (pic != NULL) {
						delete pic;
						pic = NULL;
					}
					pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
					menu = true;
					LoadLevelMusic();
				} else {
					for (short i = 0; i < 10; i++) 
						keys[i] = false;
					levelSystem->RestartLevel();
					LoadLevelMusic();
				}
				return;
			
			}
		
		}
		
		//Check Timer
		if (levelSystem->getPlayer()->getDead() == 0 && !levelSystem->getLevelFinished() && !levelSystem->getLevelStarted() && !levelSystem->getTimerOff()) {
			if (levelSystem->getLevelTimer() == -1) {
				seconds = (secondsbk + levelSystem->getLevelTime().elapsed()) / 1000;
			} else {
				seconds = levelSystem->getLevelTimer() - ((secondsbk + levelSystem->getLevelTime().elapsed()) / 1000);
			}
			minutes = seconds / 60;
			seconds %= 60;
			if (levelSystem->getLevelTimer() != -1 && minutes == 0 && seconds == 10) {
				levelSystem->getMainWindow()->playSound(SOUND_Timer10);
			}
			if (levelSystem->getLevelTimer() != -1 && minutes == 0 && seconds <= 0) {
				levelSystem->getPlayer()->died();
				secondsbk = 0;
				if (levelSystem->getPlayer()->getLives() > 0)
					levelSystem->getPlayer()->setDeadText("Time's up!");
			}
		}	
		render();
		levelSystem->setLoading(false);
	} else {
		
		if (gameend == true) {
			if (endscreen == 0) {
				tstablizer.restart();
				endscreen = 5;
			}
		}
		
		render();
	}
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::checkInput() {   
     
     if (paused) {
	     if (keys[KEY_F12] == true) {
		     levelSystem->getMainWindow()->playSound(SOUND_Pause);
		     Mix_VolumeMusic(musicVolume);
		     paused = false;
		     levelSystem->getLevelTime().restart();
		     pauseTimer.restart();
	     }
	     return;
     }
     
     levelSystem->getPlayer()->setPush(false);

     BYTE move = 0;

     ////////////////////////////////////////////////////////////////////////////////////////////
     if (keys[KEY_LEFT] == true && slide == false) {  //move left
	    // if (levelSystem->getPlayer()->getGoingRight() == true)
		//     interval = 1;
	  //   levelSystem->getPlayer()->setGoingRight(false);
	     if (interval < 7)
		     interval = interval + 1;
	     if (interval > 7)
		     interval = 7;
	     move -= interval;
     } else if (keys[KEY_RIGHT] != true && levelSystem->getPlayer()->getGoingRight() == false && slide == false) {
	     if (interval > 1) {
	//	     levelSystem->getPlayer()->setGoingRight(false);
		     move -= interval;
		     interval = interval - 1;
	     }
	     if (interval <= 1) {
		  interval = 1;
		  slide = false;
		  levelSystem->getPlayer()->setGoingRight(false);
	     }
		     
     } else if (keys[KEY_RIGHT] == true && levelSystem->getPlayer()->getGoingRight() == false && interval > 1) {
               slide = true;
	       levelSystem->getPlayer()->setGoingRight(true);
     } 
     
     if (levelSystem->getPlayer()->getGoingRight() == true && slide == true && interval > 1) {  //slide
	     levelSystem->getPlayer()->setGoingRight(true);
	     move -= interval;
	     interval = interval - 1;
	     if (interval <= 1) {
		  interval = 1;
		  slide = false;
		  levelSystem->getPlayer()->setGoingRight(true);
	     }
     }
     /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     if (keys[KEY_RIGHT] == true && slide == false) {  //move right
	  //   if (levelSystem->getPlayer()->getGoingRight() == false)
		//     interval = 1;
	    // levelSystem->getPlayer()->setGoingRight(true);
	     if (interval < 7)
		     interval = interval + 1;
	     if (interval > 7)
		     interval = 7;
	     move += interval;
	     
     } else if (keys[KEY_LEFT] != true && levelSystem->getPlayer()->getGoingRight() == true && slide == false) {
	     if (interval > 1) {
		//     levelSystem->getPlayer()->setGoingRight(true);
		     move += interval;
		     interval = interval - 1;
	     }
	     if (interval <= 1) {
		  interval = 1;
		  slide = false;
		  levelSystem->getPlayer()->setGoingRight(true);
	     }
     
     } else if (keys[KEY_LEFT] == true && levelSystem->getPlayer()->getGoingRight() == true && interval > 1) {
	     slide = true;
	     levelSystem->getPlayer()->setGoingRight(false);
     } 
     
     if (levelSystem->getPlayer()->getGoingRight() == false && slide == true && interval > 1) {
	     levelSystem->getPlayer()->setGoingRight(false);
	     move += interval;
	     interval = interval - 1;
	     if (interval <= 1) {
		  interval = 1;
		  slide = false;
		  levelSystem->getPlayer()->setGoingRight(false);
	     }
     }
     ///////////////////////////////////////////////////////////////////////////////////////////////////

     if (keys[KEY_ESCAPE] == true) {  //on Zaurus its * * [@] * *    //Cancel key - exit to main menu
	     levelSystem->getMainWindow()->playSound(SOUND_BeepMenuA);
	     if (pic != NULL) {
		     delete pic;
		     pic = NULL;
	     }
	     pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg3.jpg"));
	     inGame = true;
	     menu = true;
	     StopLevelMusic(true);
	     LoadLevelMusic();
	     keys[KEY_ESCAPE] = false;
     }
     
     if (keys[KEY_F12] == true) {
	     if (pauseTimer.elapsed() > 250) {
		     levelSystem->getMainWindow()->playSound(SOUND_Pause);
		     Mix_VolumeMusic(20);
		     paused = true;
		     secondsbk = secondsbk + levelSystem->getLevelTime().elapsed();
		     levelSystem->getLevelTime().restart();
		     pauseTimer.restart();
		     return;
	     }
     }
            
     if (keys[KEY_F33] == true || keys[KEY_ENTER] == true) {  //OK key
	     if (levelSystem->getPlayer()->getOnGround()) {
		     levelSystem->getPlayer()->setJump(true);
		     levelSystem->getMainWindow()->playSound(SOUND_Jump);
	     }
     }
                   
     if (keys[KEY_F8] == true) {
	     levelSystem->getPlayer()->setPush(true);
	     levelSystem->getMainWindow()->playSound(SOUND_PsyAttack);
     }
     
     if (keys[KEY_F13] == true) {  //on Zaurus its * * [*] * @  //push
	     levelSystem->getMainWindow()->playSound(SOUND_PsyAttack);
	     levelSystem->getPlayer()->setPush(true);
	     keys[KEY_F13] = false;
     }
   
    if (!levelSystem->getPlayer()->checkWallCollision(move)) {

    	if(!levelSystem->getPlayer()->getJump()) {
		levelSystem->getPlayer()->setY(levelSystem->getPlayer()->getY() - 20);    //Apply uplift so we can walk up slopes and things //needs to be 20 to get over last hump..check this on PDA //10 was orig
		levelSystem->getPlayer()->setOnGround(false);
	}
    	 if (levelSystem->getPlayer()->getCentered()) {
	      levelSystem->setCurrentX(levelSystem->getCurrentX() + move);                           //Appy horizontal move
	 } else {                                         //we are offset
	      
	      if (levelSystem->getPlayer()->getOffset() < 0) {                          //offset to the left
		      if (move > 0) {
			    levelSystem->getPlayer()->setOffset(levelSystem->getPlayer()->getOffset() + move);
			    if (levelSystem->getPlayer()->getOffset() >= 0) {           //we are centered again
				    levelSystem->setCurrentX(levelSystem->getCurrentX() + levelSystem->getPlayer()->getOffset());
				    levelSystem->getPlayer()->setOffset(0);
				    levelSystem->getPlayer()->setX(120-(levelSystem->getPlayer()->getWidth() >> 1));
				    levelSystem->getPlayer()->setCentered(true);
			    } else
			         levelSystem->getPlayer()->setX((120 - (levelSystem->getPlayer()->getWidth() >> 1)) + levelSystem->getPlayer()->getOffset());
		      }
	      } else {                                    //offset to the right
		      if (move < 0) {
			    levelSystem->getPlayer()->setOffset(levelSystem->getPlayer()->getOffset() + move);
			    if (levelSystem->getPlayer()->getOffset() <= 0) {           //we are centered again
				    levelSystem->setCurrentX(levelSystem->getCurrentX() + levelSystem->getPlayer()->getOffset());
				    levelSystem->getPlayer()->setOffset(0);
				    levelSystem->getPlayer()->setX(120-(levelSystem->getPlayer()->getWidth() >> 1));
				    levelSystem->getPlayer()->setCentered(true);
			    } else
			         levelSystem->getPlayer()->setX((120 - (levelSystem->getPlayer()->getWidth() >> 1)) + levelSystem->getPlayer()->getOffset());
		      }
	      }
	 }
	 
	 if(!levelSystem->getPlayer()->getJump()) {
		 for (BYTE i = 0; i < 4; i++)
			 levelSystem->getPlayer()->applyGravity(true, false);
	 }
    }
    
    if ((levelSystem->getCurrentX() < 0 && levelSystem->getPlayer()->getCentered() == true) || (levelSystem->getPlayer()->getCentered() == false && ((levelSystem->getCurrentX() + move) < 0))) {
	 
	 //offset jack on the screen.
	 levelSystem->getPlayer()->setCentered(false);
	 levelSystem->getPlayer()->setOffset(levelSystem->getPlayer()->getOffset() + (levelSystem->getCurrentX() + move));
	 levelSystem->setCurrentX(0);
	 if (levelSystem->getPlayer()->getOffset() < (-120 + (levelSystem->getPlayer()->getWidth() >> 1))) {
	      levelSystem->getPlayer()->setOffset(-120 + (levelSystem->getPlayer()->getWidth() >> 1));
	      levelSystem->getPlayer()->setX(0);
	 } else {
	      levelSystem->getPlayer()->setX((120 - (levelSystem->getPlayer()->getWidth() >> 1)) + levelSystem->getPlayer()->getOffset());
	 }
    }
    
    if ((levelSystem->getCurrentX() > (((levelSystem->getMapWidth()-8)*30)) && levelSystem->getPlayer()->getCentered() == true) || 
        (levelSystem->getPlayer()->getCentered() == false && (levelSystem->getCurrentX() + move) > (((levelSystem->getMapWidth()-8)*30)))) {
	
	//offset jack on the screen.
	if (levelSystem->getPlayer()->getCentered() == true)
		levelSystem->getPlayer()->setOffset(((levelSystem->getCurrentX()) - ((levelSystem->getMapWidth()-8)*30)));
	else
		levelSystem->getPlayer()->setOffset(levelSystem->getPlayer()->getOffset() + ((levelSystem->getCurrentX() + move) - ((levelSystem->getMapWidth()-8)*30)));
	levelSystem->getPlayer()->setCentered(false);
	
	
	levelSystem->setCurrentX(((levelSystem->getMapWidth()-8)*30));
	if (levelSystem->getPlayer()->getOffset() > (120 - (levelSystem->getPlayer()->getWidth()))) {
	      levelSystem->getPlayer()->setOffset(120 - (levelSystem->getPlayer()->getWidth()));
	      levelSystem->getPlayer()->setX(239 - levelSystem->getPlayer()->getWidth());
	 } else {
	      levelSystem->getPlayer()->setX((120 - (levelSystem->getPlayer()->getWidth() >> 1)) + levelSystem->getPlayer()->getOffset());	 
	 }
    }
    
    if (levelSystem->getPlayer()->getPush()) {
	    levelSystem->getPlayer()->pushDetect();
    }
    
    levelSystem->getPlayer()->animate(move);
    
    
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::startGame() {
	
	loadscreen = true;
	if (pic != NULL) {
		     delete pic;
		     pic = NULL;
	}
	pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg2.jpg"));
	rollBackArrows = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/rollback_",".jpg",4);
	tmprofile = new PLevelSystem::ProfileType;
	levelSystem->clearProfile(tmprofile);
	levelSystem->getSavedInfo(tmprofile);
	rollBackSelected1 = tmprofile->number-1;
	rollBackSelected2 = tmprofile->number2-1;
	rollBackSelected3 = tmprofile->number3-1;
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::quitGame() {
	
	//release the keyboard
	QPEApplication::grabKeyboard();
	QPEApplication::ungrabKeyboard();
	qApp->processEvents();
    
    	//exit
    	exit(0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::loadGame(BYTE profile, BYTE newgame) {
	
	BYTE rollback = 0;
	if (profile == 1)
		rollback = rollBackSelected1;
	else if (profile == 2)
		rollback = rollBackSelected2;
	else
		rollback = rollBackSelected3;
		
	loadscreen = false;
	selected = 0;
	if (pic != NULL) {
		     delete pic;
		     pic = NULL;
	}
	delete rollBackArrows;
	rollBackArrows = NULL;
	delete tmprofile;
	tmprofile = NULL;
	rollBackSelected1 = -1;
	rollBackSelected2 = -1;
	rollBackSelected3 = -1;
	levelSystem->setCurrentLevel(0);
	levelSystem->getPlayer()->setDead(0);
	levelSystem->getPlayer()->setLives(3);
	levelSystem->setLevelFinished(false);
	levelSystem->getPlayer()->setNumTires(0);
	levelSystem->setCurProfile(profile);
	if (newgame == 0)
		levelSystem->LoadGame(profile, rollback);
		
	//Load Map
        levelSystem->RestartLevel();
	menu = false;
	LoadLevelMusic();
    
    	for (short i = 0; i < 10; i++) {
		keys[i] = false;
	}
	secondsbk = 0;
    	
	tstablizer.restart();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::loadOptions() {
	
	if (pic != NULL) {
		     delete pic;
		     pic = NULL;
	}
	pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg4.jpg"));
	strcpy(password,"_ _ _ _ _ _");
	selected = 0;
	
	//Load Options from file
	QFile f( QDir::homeDirPath() + QString("/.jack/Options.dat") );
	if (f.exists() == false) {
		//go with defaults
		optionsMenu = true;
		return;
	}
		
        f.open( IO_ReadOnly );                      // open file for reading
        
        QDataStream s( &f );                        // serialize using f
	DINT a;
	BYTE num;
	
	//load music state
	s >> a;
	num = a;
	if (num == 1)
		MusicOn = true;
	else
		MusicOn = false;
	
	//load music slider
	s >> a;
	musicVolume = a;
	
	//load music state
	s >> a;
	num = a;
	if (num == 1)
		SoundOn = true;
	else
		SoundOn = false;
	
	//load sound slider
	s >> a;
	soundVolume = a;
	
	//loud cheats
	
	f.close();
	
	optionsMenu = true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::returnMenu() {
	
	//need to find out which menu
	optionsMenu = false;
	
	if (pic != NULL) {
		     delete pic;
		     pic = NULL;
	}
	
	if (!inGame)
		pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg.jpg"));
	else
		pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/menubg3.jpg"));
	
	//Save Options to file
	QFile f( QDir::homeDirPath() + QString("/.jack/Options.dat") );
	f.open( IO_WriteOnly );                     // open file for writing
	
	QDataStream s( &f );                        // serialize using f
	
	//save music state
	if (MusicOn == true)
		s << (DINT)1;
	else
		s << (DINT)0;
	
	//save music slider
	s << (DINT)musicVolume;
	
	//save music state
	if (SoundOn == true)
		s << (DINT)1;
	else
		s << (DINT)0;
	
	//save sound slider
	s << (DINT)soundVolume;
	
	//Save out cheats
	
	
	f.close();
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::PasswordCheck(char num) {
	
	if (selected != 3)
		return;
		
        for (int i = 0; i < 11; i+=2) {
		
		if (password[i] == '_') {
			password[i] = num;
			if (i == 0)
				strcpy(passOk,"");
			if (i == 10)
				break;
			else
				return;
		}
	}
	
	//Check to see if password is correct
	strcpy(passOk,"Invalid Password");
	
	//Clear Password
	strcpy(password,"_ _ _ _ _ _");
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::LoadLevelMusic() {
	
	if (MusicOn == false)
		return;
	if (music != NULL) {
		Mix_FreeMusic(music);
		music = NULL;
	}
	
	if (menu) {
		music = Mix_LoadMUS("/opt/QtPalmtop/sounds/jack/sounds/song4.mod");
	} else if (levelSystem->getCurrentLevel() <= 5) {
		music = Mix_LoadMUS("/opt/QtPalmtop/sounds/jack/sounds/song1.mod");
	}
	
	Mix_PlayMusic(music,-1);
	Mix_VolumeMusic(musicVolume);

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::StopLevelMusic(bool halt = false) {
	
	if (halt) {
		Mix_HaltMusic();
	} else {
		//Fade out the music
		Mix_FadeOutMusic(1500);
	}

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MainWidget::playSound(SOUNDTYPE snd) {
	
	if (SoundOn == false)
		return;
	
	Mix_PlayChannel(-1, sounds[snd], 0);
	Mix_VolumeChunk(sounds[snd],soundVolume);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*void MainWidget::record(ZSurface &bb) {
	
	QColor tcolor;
	QImage timage;
	timage.create (240, 320, 32);
	for (int i = 0; i < 320; i++) {
		for (int j = 0; j < 240; j++) {
			bb.getPixel(j, i, tcolor);
			timage.setPixel(j, i, tcolor.rgb());
		}
	}
	
	timage.save("avi/video_" + QString::number(numBitmaps) + ".bmp", "BMP", 100);
	
	numBitmaps++;
	
}
*/		   
