///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Project: Jack the Uni-Psychle
//
// Author: Tim Newell
//
// License: GPL see LICENSE in main directory or /usr/share/JUP/ if you used the installer
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef MAINWIDGET_H
#define MAINWIDGET_H
#include<qwidget.h>
#include <qtimer.h>
#include <qdatetime.h>
#include "CustomDefines.h"
#include "PLevelSystem.h"
//ZSurface includes
#include "../zsurface/ZSurface.h"
#include "../zsurface/ZFont.h"
//SDL includes
#include <SDL.h>
#include <SDL_mixer.h>
 
//Key Defines
#define KEY_LEFT 0
#define KEY_RIGHT 1
#define KEY_F12  2
#define KEY_ESCAPE 3
#define KEY_F33 4
#define KEY_ENTER 5
#define KEY_F8 8
#define KEY_F13 9

class PLevelSystem;

class MainWidget : public QWidget
{
	Q_OBJECT

public:

    	MainWidget( QWidget* parent = 0, const char* name = 0);
    	~MainWidget();
	void playSound(SOUNDTYPE snd);
	void LoadLevelMusic();
	void StopLevelMusic(bool halt = false);
	
	int secondsbk;
	bool slide;
	BYTE interval;
	
	//public menu stuff
	bool gameend;
	BYTE endscreen;
	bool menu;
	ZSurface *pic;

protected:
    	void keyPressEvent( QKeyEvent *e );
	void keyReleaseEvent( QKeyEvent *e);
	void mousePressEvent ( QMouseEvent *e );
	void paintEvent( QPaintEvent * );
	void resizeEvent(QResizeEvent *);
	void focusInEvent(QFocusEvent *);
	void focusOutEvent ( QFocusEvent * );
	void enableFullscreen();
	void checkInput();
	void render();
    
public slots:
    	void processLoop();

private:

	void startGame();
        void quitGame();
        void loadGame(BYTE profile, BYTE newgame);
	void loadOptions();
	void returnMenu();
	void PasswordCheck(char num);
	//void record(ZSurface &bb);
    
    	//vars
	PLevelSystem *levelSystem;
	bool keys[10];
	QTimer *t;
	QTime frameRateTimer;
	QTime pauseTimer;
	int timepassed;
	int timepassed2;
	bool inactive;
	QTime tstablizer;
	bool rendering;
	BYTE rollBackSelected1;
	BYTE rollBackSelected2;
	BYTE rollBackSelected3;
    
    	ZSurface *hudLives;
	ZSurface *hudTires;
	short seconds;
	BYTE minutes;
	ZSurface *pushPix;
	ZSurface *pullPix;
	bool inGame;
	bool paused;
	
    	//Menu stuff
	bool  introdone;
	bool loadscreen;
	BYTE selected;
	PLevelSystem::ProfileType *tmprofile;
	PZSurfaceArray *rollBackArrows;
	
	//Options Menu stuff
	bool optionsMenu;
	char password[12];
	char passOk[20];
	
	//ZSurface stuff
	ZSurface PBackBuffer;
	ZFont fontOne;
	ZFont fontTwo;
	
	//SDL Audio stuff
	Mix_Music *music;
	Mix_Chunk* sounds[14];
	short soundVolume;
	short musicVolume;
	bool MusicOn;
	bool SoundOn;
	
	//short fps;
	//int numBitmaps;
	//bool writeBitmap;
};



#endif

