///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Project: Jack the Uni-Psychle
//
// Author: Tim Newell
//
// License: GPL see LICENSE in main directory or /usr/share/JUP/ if you used the installer
// 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef PBASEOBJECT_H
#define PBASEOBJECT_H
#include "CustomDefines.h"
#include "PLevelSystem.h"
//ZSurface includes
#include "../zsurface/ZSurface.h"
#include "../zsurface/ZFont.h"

class PLevelSystem;

class PBaseObject {
	
public:

	PBaseObject(short x, short y, BYTE type, UBYTE radius, BYTE droptype, bool goingRight, short offset, PLevelSystem *levelSys);
	//abstract deconstructor
	virtual ~PBaseObject() {}
	
	//abstract functions
	virtual RTTI_OBJECTTYPE getType() = 0;
	virtual void render(ZSurface &backBuffer) = 0;
	virtual void processAI() = 0;
	virtual bool processDeadAI() = 0;
	virtual UBYTE getWidth() = 0;
	virtual UBYTE getHeight() = 0;
	
	//static
	static PBaseObject* CreateObject(RTTI_OBJECTTYPE ot, short x, short y, BYTE type, UBYTE radius, BYTE droptype, bool goingRight, short offset, PLevelSystem* levelSys);
	
	//accessor functions
	short getX() { return x; }
	short getY() { return y; }
	void setX(short _x) { x = _x; }
	void setY(short _y) { y = _y; }
	BYTE getSubType() { return type; }
	void setSubType(BYTE _type) { type = _type; }
	UBYTE getRadius() { return radius; }
	void setRadius(UBYTE _radius) { radius = _radius; }
	BYTE getDropType() { return droptype; }
	void setDropType(BYTE _droptype) { droptype = _droptype; }
	bool getGoingRight() { return goingRight; }
	void setGoingRight(bool _right) { goingRight = _right; }
	short getOffset() { return offset; }
	void setOffset(short _offset) { offset = _offset; }
	BYTE getCurrentAnimation() { return currentAnimation; }
	void setCurrentAnimation(BYTE _animation) { currentAnimation = _animation; }
	void incCurrentAnimation() { currentAnimation++; }
	BYTE getDead() { return dead; }
	void setDead(BYTE _dead) { dead = _dead; }
	
protected:
    		
	 short x;
	 short y;
         BYTE type;
	 UBYTE radius;
	 BYTE droptype;
	 bool goingRight;
         short offset;
	 BYTE currentAnimation;
	 BYTE dead;
	 PLevelSystem* levelSystem;
};



#endif

