#include "PLevelSystem.h"
#include "PPlayerObject.h"
#include "MainWidget.h"
#include <qfile.h>
#include <qimage.h>
#include <qdir.h>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLevelSystem::PLevelSystem(MainWidget *main) {
	
	mainWindow = main;
	gMapWidth = 8;
	gMapHeight = 8;
    	CurrentLevel = 0;
	CurrentX = 0;
	BGtileSet = NULL;
        tileSet = NULL;
        levelSet = NULL;
        collisionSet = NULL;
        tileSetCount = 0;
	objectCount = 0;
        objectSet = NULL;
	exitSign = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/exit.png"), true);
	tire = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/tire_",".png",4);
	spark = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/spark_",".png",2);
	pod = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/pod_",".png",2);
	playerImages = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/Jack/jack_",".png",18);
	player =  new PPlayerObject((240/2) - (playerImages->image(0)->width() >> 1), (320/2) - (playerImages->image(0)->height() >> 1), 0, 0, 0, true, 0, this);
        enemyImages[0] = NULL;
	enemyImages[1] = NULL;
	enemyImages[2] = NULL;
	enemyImages[3] = NULL;
	enemyImages[4] = NULL;
	enemyImages[5] = NULL;
	enemyImages[6] = NULL;
	enemyImages[7] = NULL;
	enemyImages[8] = NULL;
	enemyImages[9] = NULL;	
	levelTimer = -1;
	levelFinished = false;
	levelStarted = false;
	loading = false;
	curProfile = 0;
	
	//boss stuff
	bossImages[0] = NULL;
	bossImages[1] = NULL;
	bossImages[2] = NULL;
	bossImages[3] = NULL;
	bossImages[4] = NULL;
	
	//boss accessories
	bossBomber = NULL;
	bossLaser = NULL;
	bossBullet = NULL;
	bossTurrent = NULL;
	numTurrents = 6;
	
	//LevelNames
	levelNames[0] = "Fairgrounds";
	levelNames[1] = "Fairgrounds";
	levelNames[2] = "Roller Coaster";
	levelNames[3] = "Roller Coaster";
	levelNames[4] = "Dr. Apocalypse!";
	levelNames[5] = "Leaving...Go!";
	levelNames[6] = NULL;
	levelNames[7] = NULL;
	levelNames[8] = NULL;
	levelNames[9] = NULL;
	levelNames[10] = NULL;
	levelNames[11] = NULL;
	levelNames[12] = NULL;
	levelNames[13] = NULL;
	levelNames[14] = NULL;
	levelNames[15] = NULL;
	levelNames[16] = NULL;
	levelNames[17] = NULL;
	levelNames[18] = NULL;
	levelNames[19] = NULL;
	levelNames[20] = NULL;
	levelNames[21] = NULL;
	levelNames[22] = NULL;
	levelNames[23] = NULL;
	levelNames[24] = NULL;
	levelNames[25] = NULL;
	levelNames[26] = NULL;
	levelNames[27] = NULL;
	levelNames[28] = NULL;
	levelNames[29] = NULL;
	levelNames[30] = NULL;
	levelNames[31] = NULL;
	levelNames[32] = NULL;
	levelNames[33] = NULL;
	levelNames[34] = NULL;
	
	//cheats
	infLives = false;
	timerOff = false;
	dubs = false;
	timeTrials = false;
	levelWarp = false;
	PsychicStorm = false;
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PLevelSystem::~PLevelSystem() {
	
	//delete background arrays
	if (BGtileSet != NULL) {
		delete BGtileSet;
		BGtileSet = NULL;
	}
	
	
	//delete foreground arrays
	if (levelSet != NULL) {
		for (BYTE i = 0; i < 8; i++) {
			if (levelSet[i] != NULL) {
				delete[] levelSet[i];
				levelSet[i] = NULL;
			}
		}
	
		delete[] levelSet;
		levelSet = NULL;
	}
	
	if (tileSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (tileSet[i] != NULL) {
				delete tileSet[i];
				tileSet[i] = NULL;
			}
		}
	    
	    	delete[] tileSet;
		tileSet = NULL;
	}
	
	//Delete Collision arrays
	if (collisionSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			for (BYTE j = 0; j < 30; j++) {
				if (collisionSet[i][j] != NULL) {
					delete[] collisionSet[i][j];
					collisionSet[i][j] = NULL;
				}
			}
		}
	
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (collisionSet[i] != NULL) {
				delete[] collisionSet[i];
				collisionSet[i] = NULL;
			}
		}
	
		delete[] collisionSet;
		collisionSet = NULL;
	}
	    
	//delete the object array
	if (objectSet != NULL) {
		for (short i = 0; i < objectCount; i++) {
			
			if (objectSet[i] != NULL) {
				delete objectSet[i];
				objectSet[i] = NULL;
			}
			
		}
	
        	delete[] objectSet;
		objectSet = NULL;
	
	}
	
        if (player != NULL) {
             delete player;
	     player = NULL;
	}
	
	if (playerImages != NULL) {
		delete playerImages;
		playerImages = NULL;
	}
	     
	for (BYTE i = 0; i < 10; i++) {
		if (enemyImages[i] != NULL) {
			delete enemyImages[i];
			enemyImages[i] = NULL;
		}
	}
	
	delete exitSign;
	delete tire;
	delete spark;
	delete pod;
	
	for (BYTE i = 0; i < 5; i++)
		if (bossImages[i] != NULL) {
			delete bossImages[i];
			bossImages[i] = NULL;
		}
		
	//boss accessories
	if (bossBullet != NULL) {
		delete bossBullet;
		bossBullet = NULL;
	}
	
	
	if (bossBomber != NULL) {
		delete bossBomber;
		bossBomber = NULL;
	}
	
	if (bossLaser != NULL) {
		delete bossLaser;
		bossLaser = NULL;
	}
	
	if (bossTurrent != NULL) {
		delete bossTurrent;
		bossTurrent = NULL;
	}
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::render(ZSurface &backBuffer) {
	
	//First Lets render the background tiles
	short i = CurrentX / 30;    //number of tiles offset
	short j = CurrentX % 30;   //offset in pixels to subtract
	BYTE xtiles = 8;
	if (j > 0)
		xtiles++;
	   
	short x = 0;
	short y = 40;
	BYTE tmp = (i % 8);
	for (BYTE m = 0; m < 8; m++) {
		for (BYTE n = 0; n < xtiles; n++) {
	        
			if (tmp > 7) {
				tmp = tmp - 8;
			}
			ZSurface::bitBlit(&backBuffer,x-j,y,BGtileSet, tmp*30, m*30, 30, 30);
			if (levelSet[m][n+i] != -1)
				ZSurface::bitBlitAlpha(&backBuffer,x-j,y,tileSet[levelSet[m][n+i]]);
			x += 30;
			tmp++;
		}
		y += 30;
		x = 0;
		tmp = (i % 8);
	}
	
	//Next render the enemies
	for (short i = 0; i < objectCount; i++) {      //loop through the different objects
	
		if (objectSet[i] != NULL) {
			if (objectSet[i]->getX() >= (CurrentX-120) && objectSet[i]->getX() <= (CurrentX+360)) {
	
				objectSet[i]->render(backBuffer);
					
			}
		}
	}

	//Now draw the player on top
	player->render(backBuffer);
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::calculateAI() {
	
	//process objects AI
	for (short i = 0; i < objectCount; i++) {      //loop through the different objects
	
		if (objectSet[i] != NULL) {
			if ((CurrentLevel == 34) || objectSet[i]->getX() >= (CurrentX-120) && objectSet[i]->getX() <= (CurrentX+360)) {
				if (objectSet[i]->getDead() == 0)
					objectSet[i]->processAI();
				else {
					if (objectSet[i]->processDeadAI() == false) {
						delete objectSet[i];
						objectSet[i] = NULL;
					}
				}
			}
		}
	}
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::getSavedInfo(ProfileType *retpro) {
	
	if (getLevelWarp() == true) {
		
		retpro->number = 34;
		for (BYTE i = 0; i < retpro->number; i++) {
			retpro->level[i] = i+1;
			retpro->lives[i] = 3;
			retpro->tires[i] = 0;
		
		}
		
		retpro->number2 = 34;
		for (BYTE i = 0; i < retpro->number2; i++) {
			retpro->level2[i] = i+1;
			retpro->lives2[i] = 3;
			retpro->tires2[i] = 0;
		
		}
		
		retpro->number3 = 34;
		for (BYTE i = 0; i < retpro->number3; i++) {
			retpro->level3[i] = i+1;
			retpro->lives3[i] = 3;
			retpro->tires3[i] = 0;
		
		}
		
		return;
	}
	
	QFile f(QDir::homeDirPath() + QString("/.jack/gameInfo.dat"));
	if (f.exists() == false) {
		return;
	}
		
        f.open( IO_ReadOnly );                      // open file for reading
        
        QDataStream s( &f );                        // serialize using f
	DINT a;
	
	//Profile 1////////////////////////
	//get num of saves for rollback
	s >> a;
	retpro->number = a;
	for (BYTE i = 0; i < retpro->number; i++) {
		//get current level
		s >> a;
		retpro->level[i] = a;
	
		//get num Lives
		s >> a;
		retpro->lives[i] = a;
	
		//get num tires
		s >> a;
		retpro->tires[i] = a;
	}
	
	//Profile 2////////////////////////
	//get num of saves for rollback
	s >> a;
	retpro->number2 = a;
	for (BYTE i = 0; i < retpro->number2; i++) {
		//get current level
		s >> a;
		retpro->level2[i] = a;
	
		//get num Lives
		s >> a;
		retpro->lives2[i] = a;
	
		//get num tires
		s >> a;
		retpro->tires2[i] = a;
	}
	
	//Profile 3////////////////////////
	//get num of saves for rollback
	s >> a;
	retpro->number3 = a;
	for (BYTE i = 0; i < retpro->number3; i++) {
		//get current level
		s >> a;
		retpro->level3[i] = a;
	
		//get num Lives
		s >> a;
		retpro->lives3[i] = a;
	
		//get num tires
		s >> a;
		retpro->tires3[i] = a;
	}
		
	
	f.close();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::LoadGame(BYTE profile, BYTE rollback) {
	
	if (getLevelWarp() == true) {
		
		CurrentLevel = rollback+1;
		player->setLives(3);
		player->setNumTires(0);
		return;
	}
	
	QFile f( QDir::homeDirPath() + QString("/.jack/gameInfo.dat") );
	if (f.exists() == false) {
		return;
	}
		
        f.open( IO_ReadOnly );                      // open file for reading
        
        QDataStream s( &f );                        // serialize using f
	DINT a;
	BYTE num;
	
	switch (profile) {
		case 1:
			s >> a;
			num = a;
			if (num == 0) {
				break;
			}
			for (BYTE i = 0; i < rollback; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			
			s >> a;
			CurrentLevel = a;
			s >> a;
			player->setLives(a);
			s >> a;
			player->setNumTires(a);
			break;
		case 2:
			s >> a;
			num = a;
			for (BYTE i = 0; i < num; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			s >> a;
			num = a;
			if (num == 0) {
				break;
			}
			for (BYTE i = 0; i < rollback; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			
			s >> a;
			CurrentLevel = a;
			s >> a;
			player->setLives(a);
			s >> a;
			player->setNumTires(a);
			break;
				
		case 3:
			s >> a;
			num = a;
			for (BYTE i = 0; i < num; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			
			s >> a;
			num = a;
			for (BYTE i = 0; i < num; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			
			s >> a;
			num = a;
			if (num == 0) {
				break;
			}
			for (BYTE i = 0; i < rollback; i++) {
				s >> a;
				s >> a;
				s >> a;
			}
			
			s >> a;
			CurrentLevel = a;
			s >> a;
			player->setLives(a);
			s >> a;
			player->setNumTires(a);
			break;
	}
	
	f.close();
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::SaveGame(BYTE profile) {
	
	if (getLevelWarp() == true) {
		return;
	}
	
	if (player->getLives() == 0)
		return;
		
	ProfileType *tpro = new ProfileType;
	clearProfile(tpro);
	getSavedInfo(tpro);
	
	switch(profile) {
		case 1:
			tpro->level[CurrentLevel-1] = CurrentLevel;
			tpro->lives[CurrentLevel-1] = player->getLives();
			tpro->tires[CurrentLevel-1] = player->getNumTires();
			if (CurrentLevel > tpro->number)
				tpro->number++;
			break;
		case 2:
			tpro->level2[CurrentLevel-1] = CurrentLevel;
			tpro->lives2[CurrentLevel-1] = player->getLives();
			tpro->tires2[CurrentLevel-1] = player->getNumTires();
			if (CurrentLevel > tpro->number2)
				tpro->number2++;
			break;
		case 3:
			tpro->level3[CurrentLevel-1] = CurrentLevel;
			tpro->lives3[CurrentLevel-1] = player->getLives();
			tpro->tires3[CurrentLevel-1] = player->getNumTires();
			if (CurrentLevel > tpro->number3)
				tpro->number3++;
			break;
	}
	
	
	QFile f( QDir::homeDirPath() + QString("/.jack/gameInfo.dat") );
	f.open( IO_WriteOnly );                     // open file for writing
	
	QDataStream s( &f );                        // serialize using f
	
	//Profile 1////////////////////////
	//save out number of real profiles
	s << (DINT)tpro->number;
	
	for (BYTE i = 0; i < tpro->number; i++) {
		//save out current level
		s << (DINT)tpro->level[i];
	
		//save out num Lives
		s << (DINT)tpro->lives[i];
	
		//save out num tires
		s << (DINT)tpro->tires[i];
	}
	
	//Profile 2////////////////////////
	//save out number of real profiles
	s << (DINT)tpro->number2;
	
	for (BYTE i = 0; i < tpro->number2; i++) {
		//save out current level
		s << (DINT)tpro->level2[i];
	
		//save out num Lives
		s << (DINT)tpro->lives2[i];
	
		//save out num tires
		s << (DINT)tpro->tires2[i];
	}
	
	//Profile 3////////////////////////
	//save out number of real profiles
	s << (DINT)tpro->number3;
	
	for (BYTE i = 0; i < tpro->number3; i++) {
		//save out current level
		s << (DINT)tpro->level3[i];
	
		//save out num Lives
		s << (DINT)tpro->lives3[i];
	
		//save out num tires
		s << (DINT)tpro->tires3[i];
	}
	
	f.close();
	delete tpro;
	tpro = NULL;
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::ChangeLevels() {
	
	CurrentLevel++;
	//Save Game
	SaveGame(curProfile);
	RestartLevel();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::RestartLevel() {
	
	//delete previous
	//delete background arrays
	if (BGtileSet != NULL) {
		delete BGtileSet;
		BGtileSet = NULL;
	}
	
	
	//delete foreground arrays
	if (levelSet != NULL) {
		for (BYTE i = 0; i < 8; i++) {
			if (levelSet[i] != NULL) {
				delete[] levelSet[i];
				levelSet[i] = NULL;
			}
		}
	
		delete[] levelSet;
		levelSet = NULL;
	}
	
	if (tileSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (tileSet[i] != NULL) {
				delete tileSet[i];
				tileSet[i] = NULL;
			}
		}
	    
	    	delete[] tileSet;
		tileSet = NULL;
	}
	
	//Delete Collision arrays
	if (collisionSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			for (BYTE j = 0; j < 30; j++) {
				if (collisionSet[i][j] != NULL) {
					delete[] collisionSet[i][j];
					collisionSet[i][j] = NULL;
				}
			}
		}
	
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (collisionSet[i] != NULL) {
				delete[] collisionSet[i];
				collisionSet[i] = NULL;
			}
		}
	
		delete[] collisionSet;
		collisionSet = NULL;
	}
	    
	//delete the object array
	if (objectSet != NULL) {
		for (BYTE i = 0; i < objectCount; i++) {
			
			if (objectSet[i] != NULL) {
				delete objectSet[i];
				objectSet[i] = NULL;
			}
			
		}
	
        	delete[] objectSet;
		objectSet = NULL;
	
	}
	
	//delete enemy images
	for (BYTE i = 0; i < 10; i++) {
		if (enemyImages[i] != NULL) {
			delete enemyImages[i];
			enemyImages[i] = NULL;
		}
	}
	
	//delete boss images
	for (BYTE i = 0; i < 5; i++)
		if (bossImages[i] != NULL) {
			delete bossImages[i];
			bossImages[i] = NULL;
		}
		
		
	//delete boss accessories images
	if (bossBullet != NULL) {
		delete bossBullet;
		bossBullet = NULL;
	}
	
	if (bossBomber != NULL) {
		delete bossBomber;
		bossBomber = NULL;
	}
	
	if (bossLaser != NULL) {
		delete bossLaser;
		bossLaser = NULL;
	}
	
	if (bossTurrent != NULL) {
		delete bossTurrent;
		bossTurrent = NULL;
	}
	
	//reset vars
	tileSetCount = 0;
	CurrentX = 0;
	numTurrents = 6;
	
	player->setCentered(true);
	player->setOffset(0);
	player->setJumpedDistance(0);
	player->setJump(false);
	player->setPush(false);
	player->setOnGround(false);
	mainWindow->interval = 1;
	mainWindow->secondsbk = 0;
	
	levelTimer = -1;
	
	//Load Map
        LoadMap("/opt/QtPalmtop/pics/jack/maps/level" + QString::number(CurrentLevel+1) + ".jmf");   //need to replace levels array
	
	//reset player
        player->setX((240 >> 1) - (player->getWidth() >> 1));
	player->setY((320 >> 1) - (player->getHeight() >> 1));
	player->setCurrentAnimation(0);
        player->setGoingRight(true);
	levelStarted = true;

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::LoadMap(const char *level) {
	
	loading = true;
	
	//Load Map File
	QFile f( level );
        f.open( IO_ReadOnly );                      // open file for reading

        QDataStream s( &f );                        // serialize using f
	
	char *str;
	s >> str;
	BGtileSet = new ZSurface(QPixmap(QString("/opt/QtPalmtop/pics/jack/") + str));
	delete str;
	
	//Now lets read FG Tiles
	DINT a;
	s >> a;                                     //grab the number of tiles in the set
	tileSetCount = a;
	tileSet = new ZSurface*[tileSetCount];	
		  
	//FIll FG Tiles
	for (BYTE i = 0; i < tileSetCount; i++) {
		char *str;
		s >> str;
		tileSet[i] = new ZSurface(QPixmap(QString("/opt/QtPalmtop/pics/jack/") + str), true);
	        delete str;
	}
	
	//Now lets read the Objects
	s >> a;
	objectCount = a;
	objectSet = new PBaseObject*[objectCount];                              //num Objects
	
	
	for (short i = 0; i < objectCount; i++) {
		DINT x,y,t,u,r,d;
	        s >> x >> y >> t >> u >> r >> d;
		switch (t) {
			case 0:  //enemy
				objectSet[i] = PBaseObject::CreateObject(RTTI_Enemy, x, y, u, r, d, true, 0, this);
				
				if (u == 0) {
					if (enemyImages[0] == NULL)
						enemyImages[0] = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/Imp/imp_",".png",6);
				} else if (u == 1) {
					if (enemyImages[1] == NULL)
						enemyImages[1] = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/Moonunit/moonunit_",".png",6);
				} else if (u == 9) {
					if (enemyImages[9] == NULL)
						enemyImages[9] = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/Ispy/ispy_",".png",2);
				}
				
				
				break;
			case 3:  //tire
				objectSet[i] = PBaseObject::CreateObject(RTTI_TireSpark, x, y, 0, r, d, true, 0, this);
				break;
			case 4:  //spark
				objectSet[i] = PBaseObject::CreateObject(RTTI_TireSpark, x, y, 1, r, d, true, 0, this);
				break;
			case 5:  //pod
				objectSet[i] = PBaseObject::CreateObject(RTTI_Pod, x, y, u, r, d, true, 0, this);
				break;
			case 6:  //exit sign
				objectSet[i] = PBaseObject::CreateObject(RTTI_Exit, x, y, u, r, d, true, 0, this);
				break;
			case 7:  //boss
				objectSet[i] = PBaseObject::CreateObject(RTTI_Boss, x, y, u, r, d, false, 0, this);
				
				if (u == 0) {
					if (bossImages[0] == NULL) {
						bossImages[0] = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/tank/tank_",".png",8);   //boss number 1
						bossBullet = new PZSurfaceArray("/opt/QtPalmtop/pics/jack/images/bullet_",".png",2);
					}
				} 
				break;
			default:
				objectSet[i] = NULL;
		}
	}
	
	s >> a;
	gMapWidth = a;                                          //Map Width
	gMapHeight = 8;
	
	levelSet = new BYTE*[gMapHeight];
	
	for (BYTE i = 0; i < gMapHeight; i++) {
	     levelSet[i] = new BYTE[gMapWidth];	
	}
	
	//fill levelSet array
	for (BYTE j = 0; j < gMapHeight; j++) 
	     for (UBYTE i = 0; i < (gMapWidth); i++) {
	          s >> a;
	          levelSet[j][i] = a;
		  
	     }
	     
	s >> a;
	levelTimer = a;
	
	if (getTimeTrials() == true)
		levelTimer = 75;
		
	//read in collision info
	//Create Collision arrays
	collisionSet = new BYTE**[tileSetCount];
	
	for (BYTE i = 0; i < tileSetCount; i++) {
	     collisionSet[i] = new BYTE*[30];	
	}
	
	for (BYTE j = 0; j < tileSetCount; j++)
	     for (BYTE i = 0; i < 30; i++) 
	          collisionSet[j][i] = new BYTE[30];
		  
		  
	for (BYTE i = 0; i < tileSetCount; i++) {
				
		for (BYTE m = 0; m < 30; m++) {
			for (BYTE n = 0; n < 30; n++) {
		               s >> a;
			       collisionSet[i][m][n] = a;
			}
		}
		
	}
	
	
	
	
	f.close();
	loading = false;
	
	levelTime.restart();

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::DropObjects(PBaseObject* object) {
	
	if (object->getDropType() == 1) {   //spark
		
		AddObjectToSet(1, RTTI_TireSpark, object->getX(), object->getY(), 1, 0, 0, true, 0);
		
	} else if (object->getDropType() == 2) {   //1 tire
		
		AddObjectToSet(1, RTTI_TireSpark, object->getX(), object->getY(), 0, 0, 0, true, 0);
		
	} else if (object->getDropType() == 3) {   //H 5 tire
		
		AddObjectToSet(1, RTTI_TireSpark, object->getX()+80, object->getY()-80, 0, 0, 0, true, 0);
		AddObjectToSet(1, RTTI_TireSpark, object->getX()+40, object->getY()-40, 0, 0, 0, true, 0);
		AddObjectToSet(1, RTTI_TireSpark, object->getX()-80, object->getY()-80, 0, 0, 0, true, 0);
		AddObjectToSet(1, RTTI_TireSpark, object->getX()-40, object->getY()-40, 0, 0, 0, true, 0);
		AddObjectToSet(1, RTTI_TireSpark, object->getX(), object->getY(), 0, 0, 0, true, 0);
		
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::AddObjectToSet(BYTE num, RTTI_OBJECTTYPE ot, short x, short y, BYTE type, UBYTE radius, BYTE droptype, bool goingRight, short offset) {

	//first check to see if there is available space
	bool nonulls = true;
	for (BYTE j = 0; j < num; j++) {
		nonulls = true;
		for (short i = 0; i < objectCount; i++) {
			if (objectSet[i] == NULL) {
				objectSet[i] = PBaseObject::CreateObject(ot, x, y, type, radius, droptype, goingRight, offset, this);
				nonulls = false;
				break;
			}
		}
		
		//no more space available.. create some
		if (nonulls == true) {
			BYTE w = num - j;
 			PBaseObject **tmp = objectSet;
			objectSet = new PBaseObject*[objectCount + w];
			for (short i = 0; i < objectCount; i++) {
				objectSet[i] = tmp[i];
				tmp[i] = NULL;
			}
			delete[] tmp;
			objectCount += w;
			
			for (BYTE h = 1; h <= w; h++) 
				objectSet[objectCount-h] = PBaseObject::CreateObject(ot, x, y, type, radius, droptype, goingRight, offset, this);
			
			break;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::clearProfile(ProfileType *profile) {
	
	for (BYTE i = 0; i < 34; i++) {
		profile->level[i] = profile->lives[i] = profile->tires[i] = -1;
		profile->level2[i] = profile->lives2[i] = profile->tires2[i] = -1;
		profile->level3[i] = profile->lives3[i] = profile->tires3[i] = -1;
	}
	
	profile->number = 0;
	profile->number2 = 0;
	profile->number3 = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::gameEnd() {
	mainWindow->menu = true;
	mainWindow->gameend = true;
	mainWindow->endscreen = 0;
	mainWindow->StopLevelMusic(true);
	mainWindow->LoadLevelMusic();
	
	if (mainWindow->pic != NULL) {
		delete mainWindow->pic;
		mainWindow->pic = NULL;
	}
	mainWindow->pic = new ZSurface(QPixmap("/opt/QtPalmtop/pics/jack/images/demo1.jpg"));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PLevelSystem::cleanup() {
	
	//delete background arrays
	if (BGtileSet != NULL) {
		delete BGtileSet;
		BGtileSet = NULL;
	}
	
	
	//delete foreground arrays
	if (levelSet != NULL) {
		for (BYTE i = 0; i < 8; i++) {
			if (levelSet[i] != NULL) {
				delete[] levelSet[i];
				levelSet[i] = NULL;
			}
		}
	
		delete[] levelSet;
		levelSet = NULL;
	}
	
	if (tileSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (tileSet[i] != NULL) {
				delete tileSet[i];
				tileSet[i] = NULL;
			}
		}
	    
	    	delete[] tileSet;
		tileSet = NULL;
	}
	
	//Delete Collision arrays
	if (collisionSet != NULL) {
		for (BYTE i = 0; i < tileSetCount; i++) {
			for (BYTE j = 0; j < 30; j++) {
				if (collisionSet[i][j] != NULL) {
					delete[] collisionSet[i][j];
					collisionSet[i][j] = NULL;
				}
			}
		}
	
		for (BYTE i = 0; i < tileSetCount; i++) {
			if (collisionSet[i] != NULL) {
				delete[] collisionSet[i];
				collisionSet[i] = NULL;
			}
		}
	
		delete[] collisionSet;
		collisionSet = NULL;
	}
	    
	//delete the object array
	if (objectSet != NULL) {
		for (BYTE i = 0; i < objectCount; i++) {
			
			if (objectSet[i] != NULL) {
				delete objectSet[i];
				objectSet[i] = NULL;
			}
			
		}
	
        	delete[] objectSet;
		objectSet = NULL;
	
	}
	
	//delete enemy images
	for (BYTE i = 0; i < 10; i++) {
		if (enemyImages[i] != NULL) {
			delete enemyImages[i];
			enemyImages[i] = NULL;
		}
	}
	
	//delete boss images
	for (BYTE i = 0; i < 5; i++)
		if (bossImages[i] != NULL) {
			delete bossImages[i];
			bossImages[i] = NULL;
		}
		
		
	//delete boss accessories images
	if (bossBullet != NULL) {
		delete bossBullet;
		bossBullet = NULL;
	}
	
	if (bossBomber != NULL) {
		delete bossBomber;
		bossBomber = NULL;
	}
	
	if (bossLaser != NULL) {
		delete bossLaser;
		bossLaser = NULL;
	}
	
	if (bossTurrent != NULL) {
		delete bossTurrent;
		bossTurrent = NULL;
	}
	
}

