///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Project: Jack the Uni-Psychle
//
// Author: Tim Newell
//
// License: GPL see LICENSE in main directory or /usr/share/JUP/ if you used the installer
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef PLEVELSYSTEM_H
#define PLEVELSYSTEM_H
#include "CustomDefines.h"
#include "PBaseObject.h"
#include <qdatetime.h>
#include "PZSurfaceArray.h"

class PBaseObject;
class PPlayerObject;
class MainWidget;

class PLevelSystem {
	
public:
	
	struct ProfileType {
		BYTE level[34];
		BYTE lives[34];
		BYTE tires[34];
		BYTE number;
		BYTE level2[34];
		BYTE lives2[34];
		BYTE tires2[34];
		BYTE number2;
		BYTE level3[34];
		BYTE lives3[34];
		BYTE tires3[34];
		BYTE number3;
	};

	PLevelSystem(MainWidget* main);
	~PLevelSystem();
	
	void getSavedInfo(ProfileType *retpro);
	void LoadGame(BYTE profile, BYTE rollback);
	void SaveGame(BYTE profile);
	void ChangeLevels();
	void RestartLevel();
	void DropObjects(PBaseObject* object);
	void AddObjectToSet(BYTE num, RTTI_OBJECTTYPE ot, short x, short y, BYTE type, UBYTE radius, BYTE droptype, bool goingRight, short offset);
	void render(ZSurface &backBuffer);
	void calculateAI();
	void clearProfile(ProfileType *profile);
	void gameEnd();
	void cleanup();
	
	//  Transform global coords to local screen
	int toLocalX(int x) { return x - CurrentX; }
	int toLocalY(int y) { return y + 40; }
	
	short getCurrentX() { return CurrentX; }
	void setCurrentX(short _x) { CurrentX = _x; }
	short getObjectCount() { return objectCount; }
	void setObjectCount(short _count) { objectCount = _count; }
	short getLevelTimer() { return levelTimer; }
	void setLevelTimer(short _timer) { levelTimer = _timer; }
	PPlayerObject* getPlayer() { return player; }
	UBYTE getMapWidth() { return gMapWidth; }
	void setMapWidth(UBYTE _width) { gMapWidth = _width; }
	BYTE getMapHeight() { return gMapHeight; }
	void setMapHeight(BYTE _height) { gMapHeight = _height; }
 	BYTE getCurrentLevel() { return CurrentLevel; }
	void setCurrentLevel(BYTE _level) { CurrentLevel = _level; }
	BYTE getTileSetCount() { return tileSetCount; }
	void setTileSetCount(BYTE _count) { tileSetCount = _count; }
	BYTE getCurProfile() { return curProfile; }
	void setCurProfile(BYTE _profile) { curProfile = _profile; }
	bool getLevelFinished() { return levelFinished; }
	void setLevelFinished(bool _finished) { levelFinished = _finished; }
	bool getLoading() { return loading; }
	void setLoading(bool _loading) { loading = _loading; }
	QTime& getLevelTime() { return levelTime; }
	void setLevelTime(QTime &_time) { levelTime = _time; }
	PZSurfaceArray* getBossBullet() { return bossBullet; }
	PZSurfaceArray* getBossLaser() { return bossLaser; }
	PZSurfaceArray* getBossTurrent() { return bossTurrent; }
	PZSurfaceArray* getBossImages(BYTE _num) { return bossImages[_num]; }
	bool getLevelStarted() { return levelStarted; }
	void setLevelStarted(bool _start) { levelStarted = _start; }
	const char* getLevelText() { return levelNames[CurrentLevel]; }
	void decTurrents() { numTurrents--; }
	BYTE getTurrents() { return numTurrents; }
	MainWidget* getMainWindow() { return mainWindow; }
	
	//cheat codes
	bool getInfLives() { return infLives; }
	void setInfLives(bool _lives) { infLives = _lives; }
	bool getTimerOff() { return timerOff; }
	void setTimerOff(bool _timer) { timerOff = _timer; }
	bool getDubs() { return dubs; }
	void setDubs(bool _d) { dubs = _d; }
	bool getTimeTrials() { return timeTrials; }
	void setTimeTrials(bool _time) { timeTrials = _time; }
	bool getLevelWarp() { return levelWarp; }
	void setLevelWarp(bool _warp) { levelWarp = _warp; }
	bool getPsychicStorm() { return PsychicStorm; }
	void setPsychicStorm(bool _storm) { PsychicStorm = _storm; }
	

protected:
    
    
private:
	friend class PEnemyObject;
	friend class PPlayerObject;
	friend class PExitObject;
	friend class PPodObject;
	friend class PTireSparkObject;
	friend class PBossBomberObject;
	
	MainWidget* mainWindow;
	
	void LoadMap(const char *level);
	 
	UBYTE gMapWidth;
	BYTE gMapHeight;
    	BYTE CurrentLevel;
	short CurrentX;
	ZSurface *BGtileSet;   //Background tileset array
        ZSurface **tileSet;   //tileset array
        BYTE **levelSet;     //the level mappings into the tileset
        BYTE ***collisionSet;     //the collisions for the foreground tiles
        BYTE tileSetCount;
	short objectCount;
        PBaseObject **objectSet;
	char* levels[35];
	char* levelNames[35];
	ZSurface *exitSign;
	PZSurfaceArray* tire;
	PZSurfaceArray* spark;
	PZSurfaceArray* pod;
	PPlayerObject *player;
	PZSurfaceArray* playerImages;
        PZSurfaceArray* enemyImages[10];
	short levelTimer;
	bool levelFinished;
	bool levelStarted;
        QTime levelTime;
	bool loading;
	BYTE curProfile;
	
	//boss stuff
	PZSurfaceArray* bossImages[5];
	PZSurfaceArray* bossBullet;
	PZSurfaceArray* bossBomber;
	PZSurfaceArray* bossLaser;
	PZSurfaceArray* bossTurrent;
	BYTE numTurrents;
	
	//cheat codes
	bool infLives;
	bool timerOff;
	bool dubs;
	bool timeTrials;
	bool levelWarp;
	bool PsychicStorm;
};



#endif

